@@ -1,4 +1,6 @@
 package org.apache.lucene.document;
+
+import java.io.Serializable;
 /**
  * Copyright 2004 The Apache Software Foundation
  *
@@ -20,7 +22,7 @@ package org.apache.lucene.document;
  * what Fields get loaded on a {@link Document} by {@link org.apache.lucene.index.IndexReader#document(int,org.apache.lucene.document.FieldSelector)}
  *
  **/
-public interface FieldSelector {
+public interface FieldSelector extends Serializable {
 
   /**
    * 
