@@ -24,6 +24,8 @@ import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.FSDirectory;
 import org.apache.lucene.store.IndexInput;
 import org.apache.lucene.store.Lock;
+import org.apache.lucene.store.LockObtainFailedException;
+import org.apache.lucene.store.AlreadyClosedException;
 
 import java.io.File;
 import java.io.FileOutputStream;
@@ -48,7 +50,7 @@ import java.util.Collection;
  opened already, but it cannot be used to delete documents from the index then.
 
  @author Doug Cutting
- @version $Id: IndexReader.java 497612 2007-01-18 22:47:03Z mikemccand $
+ @version $Id: IndexReader.java 543620 2007-06-01 21:18:56Z dnaber $
 */
 public abstract class IndexReader {
 
@@ -65,6 +67,8 @@ public abstract class IndexReader {
     public static final FieldOption ALL = new FieldOption ("ALL");
     // all indexed fields
     public static final FieldOption INDEXED = new FieldOption ("INDEXED");
+    // all fields that store payloads
+    public static final FieldOption STORES_PAYLOADS = new FieldOption ("STORES_PAYLOADS");
     // all fields which are not indexed
     public static final FieldOption UNINDEXED = new FieldOption ("UNINDEXED");
     // all fields which are indexed with termvectors enables
@@ -113,7 +117,17 @@ public abstract class IndexReader {
   private Directory directory;
   private boolean directoryOwner;
   private boolean closeDirectory;
-  protected IndexFileDeleter deleter;
+  private IndexDeletionPolicy deletionPolicy;
+  private boolean closed;
+
+  /**
+   * @throws AlreadyClosedException if this IndexReader is closed
+   */
+  protected final void ensureOpen() throws AlreadyClosedException {
+    if (closed) {
+      throw new AlreadyClosedException("this IndexReader is closed");
+    }
+  }
 
   private SegmentInfos segmentInfos;
   private Lock writeLock;
@@ -126,33 +140,59 @@ public abstract class IndexReader {
   private SegmentInfos rollbackSegmentInfos;
 
   /** Returns an IndexReader reading the index in an FSDirectory in the named
-   path. */
-  public static IndexReader open(String path) throws IOException {
-    return open(FSDirectory.getDirectory(path), true);
+   path.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   * @param path the path to the index directory */
+  public static IndexReader open(String path) throws CorruptIndexException, IOException {
+    return open(FSDirectory.getDirectory(path), true, null);
   }
 
   /** Returns an IndexReader reading the index in an FSDirectory in the named
-   path. */
-  public static IndexReader open(File path) throws IOException {
-    return open(FSDirectory.getDirectory(path), true);
+   * path.
+   * @param path the path to the index directory
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   */
+  public static IndexReader open(File path) throws CorruptIndexException, IOException {
+    return open(FSDirectory.getDirectory(path), true, null);
   }
 
-  /** Returns an IndexReader reading the index in the given Directory. */
-  public static IndexReader open(final Directory directory) throws IOException {
-    return open(directory, false);
+  /** Returns an IndexReader reading the index in the given Directory.
+   * @param directory the index directory
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   */
+  public static IndexReader open(final Directory directory) throws CorruptIndexException, IOException {
+    return open(directory, false, null);
   }
 
-  private static IndexReader open(final Directory directory, final boolean closeDirectory) throws IOException {
+  /** Expert: returns an IndexReader reading the index in the given
+   * Directory, with a custom {@link IndexDeletionPolicy}.
+   * @param directory the index directory
+   * @param deletionPolicy a custom deletion policy (only used
+   *  if you use this reader to perform deletes or to set
+   *  norms); see {@link IndexWriter} for details.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   */
+  public static IndexReader open(final Directory directory, IndexDeletionPolicy deletionPolicy) throws CorruptIndexException, IOException {
+    return open(directory, false, deletionPolicy);
+  }
+
+  private static IndexReader open(final Directory directory, final boolean closeDirectory, final IndexDeletionPolicy deletionPolicy) throws CorruptIndexException, IOException {
 
     return (IndexReader) new SegmentInfos.FindSegmentsFile(directory) {
 
-      public Object doBody(String segmentFileName) throws IOException {
+      protected Object doBody(String segmentFileName) throws CorruptIndexException, IOException {
 
         SegmentInfos infos = new SegmentInfos();
         infos.read(directory, segmentFileName);
 
+        IndexReader reader;
+
         if (infos.size() == 1) {		  // index is optimized
-          return SegmentReader.get(infos, infos.info(0), closeDirectory);
+          reader = SegmentReader.get(infos, infos.info(0), closeDirectory);
         } else {
 
           // To reduce the chance of hitting FileNotFound
@@ -173,21 +213,29 @@ public abstract class IndexReader {
             }
           }
 
-          return new MultiReader(directory, infos, closeDirectory, readers);
+          reader = new MultiReader(directory, infos, closeDirectory, readers);
         }
+        reader.deletionPolicy = deletionPolicy;
+        return reader;
       }
     }.run();
   }
 
-  /** Returns the directory this index resides in. */
-  public Directory directory() { return directory; }
+  /** Returns the directory this index resides in.
+   */
+  public Directory directory() {
+    ensureOpen();
+    return directory;
+  }
 
   /**
    * Returns the time the index in the named directory was last modified.
    * Do not use this to check whether the reader is still up-to-date, use
    * {@link #isCurrent()} instead. 
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long lastModified(String directory) throws IOException {
+  public static long lastModified(String directory) throws CorruptIndexException, IOException {
     return lastModified(new File(directory));
   }
 
@@ -195,8 +243,10 @@ public abstract class IndexReader {
    * Returns the time the index in the named directory was last modified. 
    * Do not use this to check whether the reader is still up-to-date, use
    * {@link #isCurrent()} instead. 
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long lastModified(File fileDirectory) throws IOException {
+  public static long lastModified(File fileDirectory) throws CorruptIndexException, IOException {
     return ((Long) new SegmentInfos.FindSegmentsFile(fileDirectory) {
         public Object doBody(String segmentFileName) {
           return new Long(FSDirectory.fileModified(fileDirectory, segmentFileName));
@@ -208,8 +258,10 @@ public abstract class IndexReader {
    * Returns the time the index in the named directory was last modified. 
    * Do not use this to check whether the reader is still up-to-date, use
    * {@link #isCurrent()} instead. 
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long lastModified(final Directory directory2) throws IOException {
+  public static long lastModified(final Directory directory2) throws CorruptIndexException, IOException {
     return ((Long) new SegmentInfos.FindSegmentsFile(directory2) {
         public Object doBody(String segmentFileName) throws IOException {
           return new Long(directory2.fileModified(segmentFileName));
@@ -224,9 +276,10 @@ public abstract class IndexReader {
    * 
    * @param directory where the index resides.
    * @return version number.
-   * @throws IOException if segments file cannot be read
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long getCurrentVersion(String directory) throws IOException {
+  public static long getCurrentVersion(String directory) throws CorruptIndexException, IOException {
     return getCurrentVersion(new File(directory));
   }
 
@@ -237,9 +290,10 @@ public abstract class IndexReader {
    * 
    * @param directory where the index resides.
    * @return version number.
-   * @throws IOException if segments file cannot be read
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long getCurrentVersion(File directory) throws IOException {
+  public static long getCurrentVersion(File directory) throws CorruptIndexException, IOException {
     Directory dir = FSDirectory.getDirectory(directory);
     long version = getCurrentVersion(dir);
     dir.close();
@@ -253,9 +307,10 @@ public abstract class IndexReader {
    * 
    * @param directory where the index resides.
    * @return version number.
-   * @throws IOException if segments file cannot be read.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public static long getCurrentVersion(Directory directory) throws IOException {
+  public static long getCurrentVersion(Directory directory) throws CorruptIndexException, IOException {
     return SegmentInfos.readCurrentVersion(directory);
   }
 
@@ -263,17 +318,26 @@ public abstract class IndexReader {
    * Version number when this IndexReader was opened.
    */
   public long getVersion() {
+    ensureOpen();
     return segmentInfos.getVersion();
   }
 
   /**
-   * Check whether this IndexReader still works on a current version of the index.
-   * If this is not the case you will need to re-open the IndexReader to
-   * make sure you see the latest changes made to the index.
+   * Check whether this IndexReader is still using the
+   * current (i.e., most recently committed) version of the
+   * index.  If a writer has committed any changes to the
+   * index since this reader was opened, this will return
+   * <code>false</code>, in which case you must open a new
+   * IndexReader in order to see the changes.  See the
+   * description of the <a href="IndexWriter.html#autoCommit"><code>autoCommit</code></a>
+   * flag which controls when the {@link IndexWriter}
+   * actually commits changes to the index.
    * 
-   * @throws IOException
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public boolean isCurrent() throws IOException {
+  public boolean isCurrent() throws CorruptIndexException, IOException {
+    ensureOpen();
     return SegmentInfos.readCurrentVersion(directory) == segmentInfos.getVersion();
   }
 
@@ -282,7 +346,8 @@ public abstract class IndexReader {
    * @return <code>true</code> if the index is optimized; <code>false</code> otherwise
    */
   public boolean isOptimized() {
-      return segmentInfos.size() == 1 && hasDeletions() == false;
+    ensureOpen();
+    return segmentInfos.size() == 1 && hasDeletions() == false;
   }
 
   /**
@@ -363,8 +428,12 @@ public abstract class IndexReader {
   public abstract int maxDoc();
 
   /** Returns the stored fields of the <code>n</code><sup>th</sup>
-   <code>Document</code> in this index. */
-  public Document document(int n) throws IOException{
+   <code>Document</code> in this index.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   */
+  public Document document(int n) throws CorruptIndexException, IOException {
+    ensureOpen();
     return document(n, null);
   }
 
@@ -372,7 +441,7 @@ public abstract class IndexReader {
    * Get the {@link org.apache.lucene.document.Document} at the <code>n</code><sup>th</sup> position. The {@link org.apache.lucene.document.FieldSelector}
    * may be used to determine what {@link org.apache.lucene.document.Field}s to load and how they should be loaded.
    * 
-   * <b>NOTE:</b> If this Reader (more specifically, the underlying {@link FieldsReader} is closed before the lazy {@link org.apache.lucene.document.Field} is
+   * <b>NOTE:</b> If this Reader (more specifically, the underlying <code>FieldsReader</code>) is closed before the lazy {@link org.apache.lucene.document.Field} is
    * loaded an exception may be thrown.  If you want the value of a lazy {@link org.apache.lucene.document.Field} to be available after closing you must
    * explicitly load it or fetch the Document again with a new loader.
    * 
@@ -380,7 +449,8 @@ public abstract class IndexReader {
    * @param n Get the document at the <code>n</code><sup>th</sup> position
    * @param fieldSelector The {@link org.apache.lucene.document.FieldSelector} to use to determine what Fields should be loaded on the Document.  May be null, in which case all Fields will be loaded.
    * @return The stored fields of the {@link org.apache.lucene.document.Document} at the nth position
-   * @throws IOException If there is a problem reading this document
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    * 
    * @see org.apache.lucene.document.Fieldable
    * @see org.apache.lucene.document.FieldSelector
@@ -388,7 +458,7 @@ public abstract class IndexReader {
    * @see org.apache.lucene.document.LoadFirstFieldSelector
    */
   //When we convert to JDK 1.5 make this Set<String>
-  public abstract Document document(int n, FieldSelector fieldSelector) throws IOException;
+  public abstract Document document(int n, FieldSelector fieldSelector) throws CorruptIndexException, IOException;
   
   
 
@@ -402,6 +472,7 @@ public abstract class IndexReader {
   public boolean hasNorms(String field) throws IOException {
     // backward compatible implementation.
     // SegmentReader has an efficient implementation.
+    ensureOpen();
     return norms(field) != null;
   }
 
@@ -422,49 +493,75 @@ public abstract class IndexReader {
 
   /** Expert: Resets the normalization factor for the named field of the named
    * document.  The norm represents the product of the field's {@link
-   * Fieldable#setBoost(float) boost} and its {@link Similarity#lengthNorm(String,
+   * org.apache.lucene.document.Fieldable#setBoost(float) boost} and its {@link Similarity#lengthNorm(String,
    * int) length normalization}.  Thus, to preserve the length normalization
    * values when resetting this, one should base the new value upon the old.
    *
    * @see #norms(String)
    * @see Similarity#decodeNorm(byte)
+   * @throws StaleReaderException if the index has changed
+   *  since this reader was opened
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws IOException if there is a low-level IO error
    */
   public final synchronized  void setNorm(int doc, String field, byte value)
-          throws IOException{
+          throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     if(directoryOwner)
-      aquireWriteLock();
+      acquireWriteLock();
     hasChanges = true;
     doSetNorm(doc, field, value);
   }
 
   /** Implements setNorm in subclass.*/
   protected abstract void doSetNorm(int doc, String field, byte value)
-          throws IOException;
+          throws CorruptIndexException, IOException;
 
   /** Expert: Resets the normalization factor for the named field of the named
    * document.
    *
    * @see #norms(String)
    * @see Similarity#decodeNorm(byte)
+   * 
+   * @throws StaleReaderException if the index has changed
+   *  since this reader was opened
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws IOException if there is a low-level IO error
    */
   public void setNorm(int doc, String field, float value)
-          throws IOException {
+          throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     setNorm(doc, field, Similarity.encodeNorm(value));
   }
 
-  /** Returns an enumeration of all the terms in the index.
-   * The enumeration is ordered by Term.compareTo().  Each term
-   * is greater than all that precede it in the enumeration.
+  /** Returns an enumeration of all the terms in the index. The
+   * enumeration is ordered by Term.compareTo(). Each term is greater
+   * than all that precede it in the enumeration. Note that after
+   * calling terms(), {@link TermEnum#next()} must be called
+   * on the resulting enumeration before calling other methods such as
+   * {@link TermEnum#term()}.
+   * @throws IOException if there is a low-level IO error
    */
   public abstract TermEnum terms() throws IOException;
 
-  /** Returns an enumeration of all terms after a given term.
-   * The enumeration is ordered by Term.compareTo().  Each term
-   * is greater than all that precede it in the enumeration.
+  /** Returns an enumeration of all terms starting at a given term. If
+   * the given term does not exist, the enumeration is positioned at the
+   * first term greater than the supplied therm. The enumeration is
+   * ordered by Term.compareTo(). Each term is greater than all that
+   * precede it in the enumeration.
+   * @throws IOException if there is a low-level IO error
    */
   public abstract TermEnum terms(Term t) throws IOException;
 
-  /** Returns the number of documents containing the term <code>t</code>. */
+  /** Returns the number of documents containing the term <code>t</code>.
+   * @throws IOException if there is a low-level IO error
+   */
   public abstract int docFreq(Term t) throws IOException;
 
   /** Returns an enumeration of all the documents which contain
@@ -476,14 +573,18 @@ public abstract class IndexReader {
    * </ul>
    * <p>The enumeration is ordered by document number.  Each document number
    * is greater than all that precede it in the enumeration.
+   * @throws IOException if there is a low-level IO error
    */
   public TermDocs termDocs(Term term) throws IOException {
+    ensureOpen();
     TermDocs termDocs = termDocs();
     termDocs.seek(term);
     return termDocs;
   }
 
-  /** Returns an unpositioned {@link TermDocs} enumerator. */
+  /** Returns an unpositioned {@link TermDocs} enumerator.
+   * @throws IOException if there is a low-level IO error
+   */
   public abstract TermDocs termDocs() throws IOException;
 
   /** Returns an enumeration of all the documents which contain
@@ -501,30 +602,41 @@ public abstract class IndexReader {
    * <p> This positional information faciliates phrase and proximity searching.
    * <p>The enumeration is ordered by document number.  Each document number is
    * greater than all that precede it in the enumeration.
+   * @throws IOException if there is a low-level IO error
    */
   public TermPositions termPositions(Term term) throws IOException {
+    ensureOpen();
     TermPositions termPositions = termPositions();
     termPositions.seek(term);
     return termPositions;
   }
 
-  /** Returns an unpositioned {@link TermPositions} enumerator. */
+  /** Returns an unpositioned {@link TermPositions} enumerator.
+   * @throws IOException if there is a low-level IO error
+   */
   public abstract TermPositions termPositions() throws IOException;
 
   /**
    * Tries to acquire the WriteLock on this directory.
    * this method is only valid if this IndexReader is directory owner.
    * 
-   * @throws IOException If WriteLock cannot be acquired.
+   * @throws StaleReaderException if the index has changed
+   * since this reader was opened
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws IOException if there is a low-level IO error
    */
-  private void aquireWriteLock() throws IOException {
+  private void acquireWriteLock() throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     if (stale)
-      throw new IOException("IndexReader out of date and no longer valid for delete, undelete, or setNorm operations");
+      throw new StaleReaderException("IndexReader out of date and no longer valid for delete, undelete, or setNorm operations");
 
     if (writeLock == null) {
       Lock writeLock = directory.makeLock(IndexWriter.WRITE_LOCK_NAME);
       if (!writeLock.obtain(IndexWriter.WRITE_LOCK_TIMEOUT)) // obtain write lock
-        throw new IOException("Index locked for write: " + writeLock);
+        throw new LockObtainFailedException("Index locked for write: " + writeLock);
       this.writeLock = writeLock;
 
       // we have to check whether index has changed since this reader was opened.
@@ -533,7 +645,7 @@ public abstract class IndexReader {
         stale = true;
         this.writeLock.release();
         this.writeLock = null;
-        throw new IOException("IndexReader out of date and no longer valid for delete, undelete, or setNorm operations");
+        throw new StaleReaderException("IndexReader out of date and no longer valid for delete, undelete, or setNorm operations");
       }
     }
   }
@@ -545,10 +657,19 @@ public abstract class IndexReader {
    * method will result in an error.  The presence of this document may still be
    * reflected in the {@link #docFreq} statistic, though
    * this will be corrected eventually as the index is further modified.
+   *
+   * @throws StaleReaderException if the index has changed
+   * since this reader was opened
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws IOException if there is a low-level IO error
    */
-  public final synchronized void deleteDocument(int docNum) throws IOException {
+  public final synchronized void deleteDocument(int docNum) throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     if(directoryOwner)
-      aquireWriteLock();
+      acquireWriteLock();
     hasChanges = true;
     doDelete(docNum);
   }
@@ -557,7 +678,7 @@ public abstract class IndexReader {
   /** Implements deletion of the document numbered <code>docNum</code>.
    * Applications should call {@link #deleteDocument(int)} or {@link #deleteDocuments(Term)}.
    */
-  protected abstract void doDelete(int docNum) throws IOException;
+  protected abstract void doDelete(int docNum) throws CorruptIndexException, IOException;
 
 
   /** Deletes all documents that have a given <code>term</code> indexed.
@@ -567,9 +688,18 @@ public abstract class IndexReader {
    * passes it to this method.
    * See {@link #deleteDocument(int)} for information about when this deletion will 
    * become effective.
+   *
    * @return the number of documents deleted
+   * @throws StaleReaderException if the index has changed
+   *  since this reader was opened
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws IOException if there is a low-level IO error
    */
-  public final int deleteDocuments(Term term) throws IOException {
+  public final int deleteDocuments(Term term) throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     TermDocs docs = termDocs(term);
     if (docs == null) return 0;
     int n = 0;
@@ -584,16 +714,26 @@ public abstract class IndexReader {
     return n;
   }
 
-  /** Undeletes all documents currently marked as deleted in this index.*/
-  public final synchronized void undeleteAll() throws IOException{
+  /** Undeletes all documents currently marked as deleted in this index.
+   *
+   * @throws StaleReaderException if the index has changed
+   *  since this reader was opened
+   * @throws LockObtainFailedException if another writer
+   *  has this index open (<code>write.lock</code> could not
+   *  be obtained)
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
+   */
+  public final synchronized void undeleteAll() throws StaleReaderException, CorruptIndexException, LockObtainFailedException, IOException {
+    ensureOpen();
     if(directoryOwner)
-      aquireWriteLock();
+      acquireWriteLock();
     hasChanges = true;
     doUndeleteAll();
   }
 
   /** Implements actual undeleteAll() in subclass. */
-  protected abstract void doUndeleteAll() throws IOException;
+  protected abstract void doUndeleteAll() throws CorruptIndexException, IOException;
 
   /**
    * Should internally checkpoint state that will change
@@ -633,24 +773,17 @@ public abstract class IndexReader {
    * If an exception is hit, then either no changes or all
    * changes will have been committed to the index
    * (transactional semantics).
-   * 
-   * @throws IOException
+   * @throws IOException if there is a low-level IO error
    */
-  protected final synchronized void commit() throws IOException{
+  protected final synchronized void commit() throws IOException {
     if(hasChanges){
-      if (deleter == null) {
-        // In the MultiReader case, we share this deleter
-        // across all SegmentReaders:
-        setDeleter(new IndexFileDeleter(segmentInfos, directory));
-      }
       if(directoryOwner){
 
-        // Should not be necessary: no prior commit should
-        // have left pending files, so just defensive:
-        deleter.clearPendingFiles();
-
-        String oldInfoFileName = segmentInfos.getCurrentSegmentFileName();
-        String nextSegmentsFileName = segmentInfos.getNextSegmentFileName();
+        // Default deleter (for backwards compatibility) is
+        // KeepOnlyLastCommitDeleter:
+        IndexFileDeleter deleter =  new IndexFileDeleter(directory,
+                                                         deletionPolicy == null ? new KeepOnlyLastCommitDeletionPolicy() : deletionPolicy,
+                                                         segmentInfos, null);
 
         // Checkpoint the state we are about to change, in
         // case we have to roll back:
@@ -672,24 +805,16 @@ public abstract class IndexReader {
             // actually in the index):
             rollbackCommit();
 
-            // Erase any pending files that we were going to delete:
-            deleter.clearPendingFiles();
-
-            // Remove possibly partially written next
-            // segments file:
-            deleter.deleteFile(nextSegmentsFileName);
-
             // Recompute deletable files & remove them (so
             // partially written .del files, etc, are
             // removed):
-            deleter.findDeletableFiles();
-            deleter.deleteFiles();
+            deleter.refresh();
           }
         }
 
-        // Attempt to delete all files we just obsoleted:
-        deleter.deleteFile(oldInfoFileName);
-        deleter.commitPendingFiles();
+        // Have the deleter remove any now unreferenced
+        // files due to this commit:
+        deleter.checkpoint(segmentInfos, true);
 
         if (writeLock != null) {
           writeLock.release();  // release write lock
@@ -702,13 +827,6 @@ public abstract class IndexReader {
     hasChanges = false;
   }
 
-  protected void setDeleter(IndexFileDeleter deleter) {
-    this.deleter = deleter;
-  }
-  protected IndexFileDeleter getDeleter() {
-    return deleter;
-  }
-
   /** Implements commit. */
   protected abstract void doCommit() throws IOException;
 
@@ -716,12 +834,17 @@ public abstract class IndexReader {
    * Closes files associated with this index.
    * Also saves any new deletions to disk.
    * No other methods should be called after this has been called.
+   * @throws IOException if there is a low-level IO error
    */
   public final synchronized void close() throws IOException {
-    commit();
-    doClose();
-    if(closeDirectory)
-      directory.close();
+    if (!closed) {
+      commit();
+      doClose();
+      if (directoryOwner)
+        closed = true;
+      if(closeDirectory)
+        directory.close();
+    }
   }
 
   /** Implements close. */
@@ -753,7 +876,7 @@ public abstract class IndexReader {
    * Returns <code>true</code> iff the index in the named directory is
    * currently locked.
    * @param directory the directory to check for a lock
-   * @throws IOException if there is a problem with accessing the index
+   * @throws IOException if there is a low-level IO error
    */
   public static boolean isLocked(Directory directory) throws IOException {
     return
@@ -764,7 +887,7 @@ public abstract class IndexReader {
    * Returns <code>true</code> iff the index in the named directory is
    * currently locked.
    * @param directory the directory to check for a lock
-   * @throws IOException if there is a problem with accessing the index
+   * @throws IOException if there is a low-level IO error
    */
   public static boolean isLocked(String directory) throws IOException {
     Directory dir = FSDirectory.getDirectory(directory);
