@@ -27,7 +27,8 @@ import java.io.IOException;
 import java.io.PrintStream;
 import java.util.Vector;
 
-public final class SegmentInfos extends Vector {
+final class SegmentInfos extends Vector {
+  
   
   /** The file format version, a negative number. */
   /* Works since counter, the old 1st entry, is always >= 0 */
@@ -44,13 +45,15 @@ public final class SegmentInfos extends Vector {
    */
   public static final int FORMAT_LOCKLESS = -2;
 
-  /** This is the current file format written.  It adds a
-   * "hasSingleNormFile" flag into each segment info.
+  /** This format adds a "hasSingleNormFile" flag into each segment info.
    * See <a href="http://issues.apache.org/jira/browse/LUCENE-756">LUCENE-756</a>
    * for details.
    */
   public static final int FORMAT_SINGLE_NORM_FILE = -3;
 
+  /* This must always point to the most recent file format. */
+  private static final int CURRENT_FORMAT = FORMAT_SINGLE_NORM_FILE;
+  
   public int counter = 0;    // used to name new segments
   /**
    * counts how often the index has been changed by adding or deleting docs.
@@ -84,20 +87,12 @@ public final class SegmentInfos extends Vector {
       return -1;
     }
     long max = -1;
-    int prefixLen = IndexFileNames.SEGMENTS.length()+1;
     for (int i = 0; i < files.length; i++) {
       String file = files[i];
       if (file.startsWith(IndexFileNames.SEGMENTS) && !file.equals(IndexFileNames.SEGMENTS_GEN)) {
-        if (file.equals(IndexFileNames.SEGMENTS)) {
-          // Pre lock-less commits:
-          if (max == -1) {
-            max = 0;
-          }
-        } else {
-          long v = Long.parseLong(file.substring(prefixLen), Character.MAX_RADIX);
-          if (v > max) {
-            max = v;
-          }
+        long gen = generationFromSegmentsFileName(file);
+        if (gen > max) {
+          max = gen;
         }
       }
     }
@@ -113,7 +108,7 @@ public final class SegmentInfos extends Vector {
   public static long getCurrentSegmentGeneration(Directory directory) throws IOException {
     String[] files = directory.list();
     if (files == null)
-      throw new IOException("Cannot read directory " + directory);
+      throw new IOException("cannot read directory " + directory + ": list() returned null");
     return getCurrentSegmentGeneration(files);
   }
 
@@ -151,6 +146,22 @@ public final class SegmentInfos extends Vector {
                                                  lastGeneration);
   }
 
+  /**
+   * Parse the generation off the segments file name and
+   * return it.
+   */
+  public static long generationFromSegmentsFileName(String fileName) {
+    if (fileName.equals(IndexFileNames.SEGMENTS)) {
+      return 0;
+    } else if (fileName.startsWith(IndexFileNames.SEGMENTS)) {
+      return Long.parseLong(fileName.substring(1+IndexFileNames.SEGMENTS.length()),
+                            Character.MAX_RADIX);
+    } else {
+      throw new IllegalArgumentException("fileName \"" + fileName + "\" is not a segments file");
+    }
+  }
+
+
   /**
    * Get the next segments_N filename that will be written.
    */
@@ -173,26 +184,24 @@ public final class SegmentInfos extends Vector {
    *
    * @param directory -- directory containing the segments file
    * @param segmentFileName -- segment file to load
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public final void read(Directory directory, String segmentFileName) throws IOException {
+  public final void read(Directory directory, String segmentFileName) throws CorruptIndexException, IOException {
     boolean success = false;
 
     IndexInput input = directory.openInput(segmentFileName);
 
-    if (segmentFileName.equals(IndexFileNames.SEGMENTS)) {
-      generation = 0;
-    } else {
-      generation = Long.parseLong(segmentFileName.substring(1+IndexFileNames.SEGMENTS.length()),
-                                  Character.MAX_RADIX);
-    }
+    generation = generationFromSegmentsFileName(segmentFileName);
+
     lastGeneration = generation;
 
     try {
       int format = input.readInt();
       if(format < 0){     // file contains explicit format info
         // check that it is a format we can understand
-        if (format < FORMAT_SINGLE_NORM_FILE)
-          throw new IOException("Unknown format version: " + format);
+        if (format < CURRENT_FORMAT)
+          throw new CorruptIndexException("Unknown format version: " + format);
         version = input.readLong(); // read version
         counter = input.readInt(); // read counter
       }
@@ -224,14 +233,16 @@ public final class SegmentInfos extends Vector {
   /**
    * This version of read uses the retry logic (for lock-less
    * commits) to find the right segments file to load.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
-  public final void read(Directory directory) throws IOException {
+  public final void read(Directory directory) throws CorruptIndexException, IOException {
 
     generation = lastGeneration = -1;
 
     new FindSegmentsFile(directory) {
 
-      public Object doBody(String segmentFileName) throws IOException {
+      protected Object doBody(String segmentFileName) throws CorruptIndexException, IOException {
         read(directory, segmentFileName);
         return null;
       }
@@ -251,8 +262,10 @@ public final class SegmentInfos extends Vector {
 
     IndexOutput output = directory.createOutput(segmentFileName);
 
+    boolean success = false;
+
     try {
-      output.writeInt(FORMAT_SINGLE_NORM_FILE); // write FORMAT
+      output.writeInt(CURRENT_FORMAT); // write FORMAT
       output.writeLong(++version); // every write changes
                                    // the index
       output.writeInt(counter); // write counter
@@ -262,7 +275,16 @@ public final class SegmentInfos extends Vector {
       }         
     }
     finally {
-      output.close();
+      try {
+        output.close();
+        success = true;
+      } finally {
+        if (!success) {
+          // Try not to leave a truncated segments_N file in
+          // the index:
+          directory.deleteFile(segmentFileName);
+        }
+      }
     }
 
     try {
@@ -301,15 +323,20 @@ public final class SegmentInfos extends Vector {
   public long getVersion() {
     return version;
   }
+  public long getGeneration() {
+    return generation;
+  }
 
   /**
    * Current version number from segments file.
+   * @throws CorruptIndexException if the index is corrupt
+   * @throws IOException if there is a low-level IO error
    */
   public static long readCurrentVersion(Directory directory)
-    throws IOException {
+    throws CorruptIndexException, IOException {
 
     return ((Long) new FindSegmentsFile(directory) {
-        public Object doBody(String segmentFileName) throws IOException {
+        protected Object doBody(String segmentFileName) throws CorruptIndexException, IOException {
 
           IndexInput input = directory.openInput(segmentFileName);
 
@@ -318,8 +345,8 @@ public final class SegmentInfos extends Vector {
           try {
             format = input.readInt();
             if(format < 0){
-              if (format < FORMAT_SINGLE_NORM_FILE)
-                throw new IOException("Unknown format version: " + format);
+              if (format < CURRENT_FORMAT)
+                throw new CorruptIndexException("Unknown format version: " + format);
               version = input.readLong(); // read version
             }
           }
@@ -436,7 +463,7 @@ public final class SegmentInfos extends Vector {
       this.directory = directory;
     }
 
-    public Object run() throws IOException {
+    public Object run() throws CorruptIndexException, IOException {
       String segmentFileName = null;
       long lastGen = -1;
       long gen = 0;
@@ -471,8 +498,12 @@ public final class SegmentInfos extends Vector {
         if (0 == method) {
           if (directory != null) {
             files = directory.list();
+            if (files == null)
+              throw new FileNotFoundException("cannot read directory " + directory + ": list() returned null");
           } else {
             files = fileDirectory.list();
+            if (files == null)
+              throw new FileNotFoundException("cannot read directory " + fileDirectory + ": list() returned null");
           }
 
           gen = getCurrentSegmentGeneration(files);
@@ -482,7 +513,7 @@ public final class SegmentInfos extends Vector {
             for(int i=0;i<files.length;i++) {
               s += " " + files[i];
             }
-            throw new FileNotFoundException("no segments* file found: files:" + s);
+            throw new FileNotFoundException("no segments* file found in " + directory + ": files:" + s);
           }
         }
 
@@ -624,5 +655,6 @@ public final class SegmentInfos extends Vector {
      * during the processing that could have been caused by
      * a writer committing.
      */
-    protected abstract Object doBody(String segmentFileName) throws IOException;}
+    protected abstract Object doBody(String segmentFileName) throws CorruptIndexException, IOException;
+  }
 }
