@@ -1,11 +1,12 @@
 package org.apache.lucene.search;
 
 /**
- * Copyright 2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -25,10 +26,23 @@ public class TopDocs implements java.io.Serializable {
   public int totalHits;
   /** Expert: The top hits for the query. */
   public ScoreDoc[] scoreDocs;
-
+  /** Expert: Stores the maximum score value encountered, needed for normalizing. */
+  private float maxScore;
+  
+  /** Expert: Returns the maximum score value encountered. */
+  public float getMaxScore() {
+      return maxScore;
+  }
+  
+  /** Expert: Sets the maximum score value encountered. */
+  public void setMaxScore(float maxScore) {
+      this.maxScore=maxScore;
+  }
+  
   /** Expert: Constructs a TopDocs.*/
-  TopDocs(int totalHits, ScoreDoc[] scoreDocs) {
+  TopDocs(int totalHits, ScoreDoc[] scoreDocs, float maxScore) {
     this.totalHits = totalHits;
     this.scoreDocs = scoreDocs;
+    this.maxScore = maxScore;
   }
 }
