@@ -239,7 +239,7 @@ public class FSDirectory extends Directory {
     if (directory.exists()) {
       String[] files = directory.list(IndexFileNameFilter.getFilter());            // clear old files
       if (files == null)
-        throw new IOException("Cannot read directory " + directory.getAbsolutePath());
+        throw new IOException("cannot read directory " + directory.getAbsolutePath() + ": list() returned null");
       for (int i = 0; i < files.length; i++) {
         File file = new File(directory, files[i]);
         if (!file.delete())
@@ -291,6 +291,12 @@ public class FSDirectory extends Directory {
           } catch (ClassCastException e) {
             throw new IOException("unable to cast LockClass " + lockClassName + " instance to a LockFactory");
           }
+
+          if (lockFactory instanceof NativeFSLockFactory) {
+            ((NativeFSLockFactory) lockFactory).setLockDir(path);
+          } else if (lockFactory instanceof SimpleFSLockFactory) {
+            ((SimpleFSLockFactory) lockFactory).setLockDir(path);
+          }
         } else {
           // Our default lock is SimpleFSLockFactory;
           // default lockDir is our index directory:
@@ -429,11 +435,16 @@ public class FSDirectory extends Directory {
     return new FSIndexOutput(file);
   }
 
-  /** Returns a stream reading an existing file. */
+  // Inherit javadoc
   public IndexInput openInput(String name) throws IOException {
     return new FSIndexInput(new File(directory, name));
   }
 
+  // Inherit javadoc
+  public IndexInput openInput(String name, int bufferSize) throws IOException {
+    return new FSIndexInput(new File(directory, name), bufferSize);
+  }
+
   /**
    * So we can do some byte-to-hexchar conversion below
    */
@@ -481,126 +492,129 @@ public class FSDirectory extends Directory {
   public String toString() {
     return this.getClass().getName() + "@" + directory;
   }
-}
 
+  protected static class FSIndexInput extends BufferedIndexInput {
+  
+    private static class Descriptor extends RandomAccessFile {
+      // remember if the file is open, so that we don't try to close it
+      // more than once
+      private boolean isOpen;
+      long position;
+      final long length;
+      
+      public Descriptor(File file, String mode) throws IOException {
+        super(file, mode);
+        isOpen=true;
+        length=length();
+      }
+  
+      public void close() throws IOException {
+        if (isOpen) {
+          isOpen=false;
+          super.close();
+        }
+      }
+  
+      protected void finalize() throws Throwable {
+        try {
+          close();
+        } finally {
+          super.finalize();
+        }
+      }
+    }
+  
+    private final Descriptor file;
+    boolean isClone;
+  
+    public FSIndexInput(File path) throws IOException {
+      this(path, BufferedIndexInput.BUFFER_SIZE);
+    }
+  
+    public FSIndexInput(File path, int bufferSize) throws IOException {
+      super(bufferSize);
+      file = new Descriptor(path, "r");
+    }
+  
+    /** IndexInput methods */
+    protected void readInternal(byte[] b, int offset, int len)
+         throws IOException {
+      synchronized (file) {
+        long position = getFilePointer();
+        if (position != file.position) {
+          file.seek(position);
+          file.position = position;
+        }
+        int total = 0;
+        do {
+          int i = file.read(b, offset+total, len-total);
+          if (i == -1)
+            throw new IOException("read past EOF");
+          file.position += i;
+          total += i;
+        } while (total < len);
+      }
+    }
+  
+    public void close() throws IOException {
+      // only close the file if this is not a clone
+      if (!isClone) file.close();
+    }
+  
+    protected void seekInternal(long position) {
+    }
+  
+    public long length() {
+      return file.length;
+    }
+  
+    public Object clone() {
+      FSIndexInput clone = (FSIndexInput)super.clone();
+      clone.isClone = true;
+      return clone;
+    }
+  
+    /** Method used for testing. Returns true if the underlying
+     *  file descriptor is valid.
+     */
+    boolean isFDValid() throws IOException {
+      return file.getFD().valid();
+    }
+  }
 
-class FSIndexInput extends BufferedIndexInput {
-
-  private static class Descriptor extends RandomAccessFile {
+  protected static class FSIndexOutput extends BufferedIndexOutput {
+    RandomAccessFile file = null;
+  
     // remember if the file is open, so that we don't try to close it
     // more than once
     private boolean isOpen;
-    long position;
-    final long length;
-    
-    public Descriptor(File file, String mode) throws IOException {
-      super(file, mode);
-      isOpen=true;
-      length=length();
+  
+    public FSIndexOutput(File path) throws IOException {
+      file = new RandomAccessFile(path, "rw");
+      isOpen = true;
+    }
+  
+    /** output methods: */
+    public void flushBuffer(byte[] b, int offset, int size) throws IOException {
+      file.write(b, offset, size);
     }
-
     public void close() throws IOException {
+      // only close the file if it has not been closed yet
       if (isOpen) {
-        isOpen=false;
         super.close();
+        file.close();
+        isOpen = false;
       }
     }
-
-    protected void finalize() throws Throwable {
-      try {
-        close();
-      } finally {
-        super.finalize();
-      }
+  
+    /** Random-access methods */
+    public void seek(long pos) throws IOException {
+      super.seek(pos);
+      file.seek(pos);
     }
-  }
-
-  private final Descriptor file;
-  boolean isClone;
-
-  public FSIndexInput(File path) throws IOException {
-    file = new Descriptor(path, "r");
-  }
-
-  /** IndexInput methods */
-  protected void readInternal(byte[] b, int offset, int len)
-       throws IOException {
-    synchronized (file) {
-      long position = getFilePointer();
-      if (position != file.position) {
-        file.seek(position);
-        file.position = position;
-      }
-      int total = 0;
-      do {
-        int i = file.read(b, offset+total, len-total);
-        if (i == -1)
-          throw new IOException("read past EOF");
-        file.position += i;
-        total += i;
-      } while (total < len);
+    public long length() throws IOException {
+      return file.length();
     }
-  }
-
-  public void close() throws IOException {
-    // only close the file if this is not a clone
-    if (!isClone) file.close();
-  }
-
-  protected void seekInternal(long position) {
-  }
-
-  public long length() {
-    return file.length;
-  }
-
-  public Object clone() {
-    FSIndexInput clone = (FSIndexInput)super.clone();
-    clone.isClone = true;
-    return clone;
-  }
-
-  /** Method used for testing. Returns true if the underlying
-   *  file descriptor is valid.
-   */
-  boolean isFDValid() throws IOException {
-    return file.getFD().valid();
+  
   }
 }
-
-
-class FSIndexOutput extends BufferedIndexOutput {
-  RandomAccessFile file = null;
-
-  // remember if the file is open, so that we don't try to close it
-  // more than once
-  private boolean isOpen;
-
-  public FSIndexOutput(File path) throws IOException {
-    file = new RandomAccessFile(path, "rw");
-    isOpen = true;
-  }
-
-  /** output methods: */
-  public void flushBuffer(byte[] b, int size) throws IOException {
-    file.write(b, 0, size);
-  }
-  public void close() throws IOException {
-    // only close the file if it has not been closed yet
-    if (isOpen) {
-      super.close();
-      file.close();
-      isOpen = false;
-    }
-  }
-
-  /** Random-access methods */
-  public void seek(long pos) throws IOException {
-    super.seek(pos);
-    file.seek(pos);
-  }
-  public long length() throws IOException {
-    return file.length();
-  }
-
-}
