@@ -25,6 +25,8 @@ import junit.textui.TestRunner;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.RAMDirectory;
 import org.apache.lucene.store.FSDirectory;
+import org.apache.lucene.store.LockObtainFailedException;
+import org.apache.lucene.store.AlreadyClosedException;
 import org.apache.lucene.analysis.standard.StandardAnalyzer;
 import org.apache.lucene.analysis.WhitespaceAnalyzer;
 import org.apache.lucene.document.Document;
@@ -60,7 +62,7 @@ public class TestIndexReader extends TestCase
 
     public void testIsCurrent() throws Exception
     {
-      RAMDirectory d = new RAMDirectory();
+      RAMDirectory d = new MockRAMDirectory();
       IndexWriter writer = new IndexWriter(d, new StandardAnalyzer(), true);
       addDocumentWithFields(writer);
       writer.close();
@@ -78,6 +80,7 @@ public class TestIndexReader extends TestCase
       writer.close();
       assertFalse(reader.isCurrent());
       reader.close();
+      d.close();
     }
 
     /**
@@ -86,7 +89,7 @@ public class TestIndexReader extends TestCase
      */
     public void testGetFieldNames() throws Exception
     {
-        RAMDirectory d = new RAMDirectory();
+        RAMDirectory d = new MockRAMDirectory();
         // set up writer
         IndexWriter writer = new IndexWriter(d, new StandardAnalyzer(), true);
         addDocumentWithFields(writer);
@@ -98,6 +101,7 @@ public class TestIndexReader extends TestCase
         assertTrue(fieldNames.contains("text"));
         assertTrue(fieldNames.contains("unindexed"));
         assertTrue(fieldNames.contains("unstored"));
+        reader.close();
         // add more documents
         writer = new IndexWriter(d, new StandardAnalyzer(), false);
         // want to get some more segments here
@@ -172,7 +176,8 @@ public class TestIndexReader extends TestCase
         fieldNames = reader.getFieldNames(IndexReader.FieldOption.TERMVECTOR_WITH_POSITION_OFFSET);
         assertEquals(1, fieldNames.size());    // 4 fields are indexed with term vectors
         assertTrue(fieldNames.contains("tvpositionoffset"));
-        
+        reader.close();
+        d.close();
     }
 
 
@@ -204,7 +209,7 @@ public class TestIndexReader extends TestCase
 
     public void testBasicDelete() throws IOException
     {
-        Directory dir = new RAMDirectory();
+        Directory dir = new MockRAMDirectory();
 
         IndexWriter writer = null;
         IndexReader reader = null;
@@ -223,6 +228,7 @@ public class TestIndexReader extends TestCase
         reader = IndexReader.open(dir);
         assertEquals("first docFreq", 100, reader.docFreq(searchTerm));
         assertTermDocsCount("first reader", reader, searchTerm, 100);
+        reader.close();
 
         // DELETE DOCUMENTS CONTAINING TERM: aaa
         int deleted = 0;
@@ -243,6 +249,98 @@ public class TestIndexReader extends TestCase
         assertEquals("deleted docFreq", 100, reader.docFreq(searchTerm));
         assertTermDocsCount("deleted termDocs", reader, searchTerm, 0);
         reader.close();
+        reader2.close();
+        dir.close();
+    }
+
+    // Make sure attempts to make changes after reader is
+    // closed throws IOException:
+    public void testChangesAfterClose() throws IOException
+    {
+        Directory dir = new RAMDirectory();
+
+        IndexWriter writer = null;
+        IndexReader reader = null;
+        Term searchTerm = new Term("content", "aaa");
+
+        //  add 11 documents with term : aaa
+        writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
+        for (int i = 0; i < 11; i++)
+        {
+            addDoc(writer, searchTerm.text());
+        }
+        writer.close();
+
+        reader = IndexReader.open(dir);
+
+        // Close reader:
+        reader.close();
+
+        // Then, try to make changes:
+        try {
+          reader.deleteDocument(4);
+          fail("deleteDocument after close failed to throw IOException");
+        } catch (AlreadyClosedException e) {
+          // expected
+        }
+
+        try {
+          reader.setNorm(5, "aaa", 2.0f);
+          fail("setNorm after close failed to throw IOException");
+        } catch (AlreadyClosedException e) {
+          // expected
+        }
+
+        try {
+          reader.undeleteAll();
+          fail("undeleteAll after close failed to throw IOException");
+        } catch (AlreadyClosedException e) {
+          // expected
+        }
+    }
+
+    // Make sure we get lock obtain failed exception with 2 writers:
+    public void testLockObtainFailed() throws IOException
+    {
+        Directory dir = new RAMDirectory();
+
+        IndexWriter writer = null;
+        IndexReader reader = null;
+        Term searchTerm = new Term("content", "aaa");
+
+        //  add 11 documents with term : aaa
+        writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
+        for (int i = 0; i < 11; i++)
+        {
+            addDoc(writer, searchTerm.text());
+        }
+
+        // Create reader:
+        reader = IndexReader.open(dir);
+
+        // Try to make changes
+        try {
+          reader.deleteDocument(4);
+          fail("deleteDocument should have hit LockObtainFailedException");
+        } catch (LockObtainFailedException e) {
+          // expected
+        }
+
+        try {
+          reader.setNorm(5, "aaa", 2.0f);
+          fail("setNorm should have hit LockObtainFailedException");
+        } catch (LockObtainFailedException e) {
+          // expected
+        }
+
+        try {
+          reader.undeleteAll();
+          fail("undeleteAll should have hit LockObtainFailedException");
+        } catch (LockObtainFailedException e) {
+          // expected
+        }
+        writer.close();
+        reader.close();
     }
 
     // Make sure you can set norms & commit even if a reader
@@ -295,6 +393,38 @@ public class TestIndexReader extends TestCase
     }
 
 
+    // Make sure you can set norms & commit, and there are
+    // no extra norms files left:
+    public void testWritingNormsNoReader() throws IOException
+    {
+        Directory dir = new MockRAMDirectory();
+        IndexWriter writer = null;
+        IndexReader reader = null;
+        Term searchTerm = new Term("content", "aaa");
+
+        //  add 1 documents with term : aaa
+        writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
+        writer.setUseCompoundFile(false);
+        addDoc(writer, searchTerm.text());
+        writer.close();
+
+        //  now open reader & set norm for doc 0 (writes to
+        //  _0_1.s0)
+        reader = IndexReader.open(dir);
+        reader.setNorm(0, "content", (float) 2.0);
+        reader.close();
+        
+        //  now open reader again & set norm for doc 0 (writes to _0_2.s0)
+        reader = IndexReader.open(dir);
+        reader.setNorm(0, "content", (float) 2.0);
+        reader.close();
+        assertFalse("failed to remove first generation norms file on writing second generation",
+                    dir.fileExists("_0_1.s0"));
+        
+        dir.close();
+    }
+
+
     public void testDeleteReaderWriterConflictUnoptimized() throws IOException{
       deleteReaderWriterConflict(false);
     }
@@ -371,7 +501,7 @@ public class TestIndexReader extends TestCase
         try {
             deleted = reader.deleteDocuments(searchTerm);
             fail("Delete allowed on an index reader with stale segment information");
-        } catch (IOException e) {
+        } catch (StaleReaderException e) {
             /* success */
         }
 
@@ -438,7 +568,7 @@ public class TestIndexReader extends TestCase
 
     public void testLastModified() throws IOException {
       assertFalse(IndexReader.indexExists("there_is_no_such_index"));
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       assertFalse(IndexReader.indexExists(dir));
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
@@ -457,11 +587,12 @@ public class TestIndexReader extends TestCase
       reader = IndexReader.open(dir);
       assertTrue("old lastModified is " + version + "; new lastModified is " + IndexReader.lastModified(dir), version <= IndexReader.lastModified(dir));
       reader.close();
+      dir.close();
     }
 
     public void testVersion() throws IOException {
       assertFalse(IndexReader.indexExists("there_is_no_such_index"));
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       assertFalse(IndexReader.indexExists(dir));
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
@@ -480,10 +611,11 @@ public class TestIndexReader extends TestCase
       reader = IndexReader.open(dir);
       assertTrue("old version is " + version + "; new version is " + IndexReader.getCurrentVersion(dir), version < IndexReader.getCurrentVersion(dir));
       reader.close();
+      dir.close();
     }
 
     public void testLock() throws IOException {
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
       writer.close();
@@ -499,10 +631,11 @@ public class TestIndexReader extends TestCase
       reader.deleteDocument(0);
       reader.close();
       writer.close();
+      dir.close();
     }
 
     public void testUndeleteAll() throws IOException {
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
       addDocumentWithFields(writer);
@@ -515,10 +648,11 @@ public class TestIndexReader extends TestCase
       reader = IndexReader.open(dir);
       assertEquals(2, reader.numDocs());	// nothing has really been deleted thanks to undeleteAll()
       reader.close();
+      dir.close();
     }
 
     public void testUndeleteAllAfterClose() throws IOException {
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
       addDocumentWithFields(writer);
@@ -531,10 +665,11 @@ public class TestIndexReader extends TestCase
       reader.undeleteAll();
       assertEquals(2, reader.numDocs());	// nothing has really been deleted thanks to undeleteAll()
       reader.close();
+      dir.close();
     }
 
     public void testUndeleteAllAfterCloseThenReopen() throws IOException {
-      Directory dir = new RAMDirectory();
+      Directory dir = new MockRAMDirectory();
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDocumentWithFields(writer);
       addDocumentWithFields(writer);
@@ -549,6 +684,7 @@ public class TestIndexReader extends TestCase
       reader = IndexReader.open(dir);
       assertEquals(2, reader.numDocs());	// nothing has really been deleted thanks to undeleteAll()
       reader.close();
+      dir.close();
     }
 
     public void testDeleteReaderReaderConflictUnoptimized() throws IOException{
@@ -571,7 +707,7 @@ public class TestIndexReader extends TestCase
       int END_COUNT = 144;
       
       // First build up a starting index:
-      RAMDirectory startDir = new RAMDirectory();
+      RAMDirectory startDir = new MockRAMDirectory();
       IndexWriter writer = new IndexWriter(startDir, new WhitespaceAnalyzer(), true);
       for(int i=0;i<157;i++) {
         Document d = new Document();
@@ -667,9 +803,7 @@ public class TestIndexReader extends TestCase
           String[] startFiles = dir.list();
           SegmentInfos infos = new SegmentInfos();
           infos.read(dir);
-          IndexFileDeleter d = new IndexFileDeleter(infos, dir);
-          d.findDeletableFiles();
-          d.deleteFiles();
+          IndexFileDeleter d = new IndexFileDeleter(dir, new KeepOnlyLastCommitDeletionPolicy(), infos, null);
           String[] endFiles = dir.list();
 
           Arrays.sort(startFiles);
@@ -752,10 +886,12 @@ public class TestIndexReader extends TestCase
         // Try again with 10 more bytes of free space:
         diskFree += 10;
       }
+
+      startDir.close();
     }
 
     public void testDocsOutOfOrderJIRA140() throws IOException {
-      Directory dir = new RAMDirectory();      
+      Directory dir = new MockRAMDirectory();      
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       for(int i=0;i<11;i++) {
         addDoc(writer, "aaa");
@@ -790,11 +926,12 @@ public class TestIndexReader extends TestCase
       if (!gotException) {
         fail("delete of out-of-bounds doc number failed to hit exception");
       }
+      dir.close();
     }
 
     public void testExceptionReleaseWriteLockJIRA768() throws IOException {
 
-      Directory dir = new RAMDirectory();      
+      Directory dir = new MockRAMDirectory();      
       IndexWriter writer  = new IndexWriter(dir, new WhitespaceAnalyzer(), true);
       addDoc(writer, "aaa");
       writer.close();
@@ -822,6 +959,7 @@ public class TestIndexReader extends TestCase
       if (IndexReader.isLocked(dir)) {
         fail("write lock is still held after close");
       }
+      dir.close();
     }
 
     private String arrayToString(String[] l) {
@@ -835,6 +973,28 @@ public class TestIndexReader extends TestCase
       return s;
     }
 
+    public void testOpenReaderAfterDelete() throws IOException {
+      File dirFile = new File(System.getProperty("tempDir"),
+                          "deletetest");
+      Directory dir = FSDirectory.getDirectory(dirFile);
+      try {
+        IndexReader reader = IndexReader.open(dir);
+        fail("expected FileNotFoundException");
+      } catch (FileNotFoundException e) {
+        // expected
+      }
+
+      dirFile.delete();
+
+      // Make sure we still get a CorruptIndexException (not NPE):
+      try {
+        IndexReader reader = IndexReader.open(dir);
+        fail("expected FileNotFoundException");
+      } catch (FileNotFoundException e) {
+        // expected
+      }
+    }
+
     private void deleteReaderReaderConflict(boolean optimize) throws IOException
     {
         Directory dir = getDirectory();
